from categories.admin import CategoryAdminForm
from categories.base import CategoryBaseAdmin


class CategoryAdmin(CategoryBaseAdmin):
    form = CategoryAdminForm
    list_display = ("name", "alternate_title", "active")
    fieldsets = (
        (None, {"fields": ("parent", "name", "thumbnail", "active")}),
        (
            "Meta Data",
            {
                "fields": ("alternate_title", "alternate_url", "description", "meta_keywords", "meta_extra"),
                "classes": ("collapse",),
            },
        ),
        (
            "Advanced",
            {
                "fields": ("order", "slug"),
                "classes": ("collapse",),
            },
        ),
    )
