# Generated by Django 1.10.5 on 2017-02-17 11:11
from __future__ import unicode_literals

import django.db.models.manager
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("categories", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="category",
            managers=[
                ("tree", django.db.models.manager.Manager()),
            ],
        ),
        migrations.AlterField(
            model_name="categoryrelation",
            name="relation_type",
            field=models.CharField(
                blank=True,
                help_text="A generic text field to tag a relation, like 'leadphoto'.",
                max_length=200,
                null=True,
                verbose_name="relation type",
            ),
        ),
    ]
